@@ -37,36 +37,32 @@
 #'
 #' @examples
 #' if (require("igraph")) {
-#'   graph <- igraph::graph.famous("zachary")
+#'   graph <- igraph::make_graph("zachary")
 #'   membership <- cluster(graph, max_threads = 2)
 #' }
 cluster <- function(graph, discard_transient = 3, independent_runs = 10,
                     max_threads = 0, seed = 0, target_clusters = 0,
                     target_partitions = 5, subcluster = 1, min_clust = 5,
                     verbose = FALSE, is_directed = "detect") {
-  adj <- se2_as_matrix_i(graph)
-
-  if (is_directed == "detect") {
-    is_directed <- adj$is_directed
-  }
+  adj <- se2_as_matrix_i(graph, is_directed)
 
   if (seed == 0) {
     seed <- sample.int(9999, 1)
   }
 
-  if (subcluster > 1) {
-    membership <- matrix(as.integer(0), nrow = subcluster, ncol = adj$n_nodes)
-  } else {
-    membership <- integer(adj$n_nodes)
-  }
-
-  .C(
+  membership <- .Call(
     C_speakeasy2, as.integer(adj$se2_i), as.integer(adj$se2_p),
     as.double(adj$values), as.integer(adj$n_nodes),
     as.integer(discard_transient), as.integer(independent_runs),
     as.integer(max_threads), as.integer(seed), as.integer(target_clusters),
     as.integer(target_partitions), as.integer(subcluster),
-    as.integer(min_clust), as.logical(verbose), as.logical(is_directed),
-    membership = membership
-  )$membership
+    as.integer(min_clust), as.logical(verbose), as.logical(adj$is_directed),
+    PACKAGE = "speakeasyR"
+  )
+
+  if (subcluster > 1) {
+    matrix(membership, nrow = subcluster, ncol = adj$n_nodes)
+  } else {
+    membership
+  }
 }
