@@ -250,13 +250,14 @@ static void* se2_thread_bootstrap(void* parameters)
   for (igraph_integer_t run_i = p->tid; run_i < independent_runs;
        run_i += n_threads) {
     *p->run_i = run_i;
-    igraph_rng_t rng, *old_rng;
+    igraph_rng_t rng, old_rng;
     igraph_integer_t partition_offset = run_i * p->opts->target_partitions;
     igraph_vector_int_t ic_store;
 
-    old_rng = se2_rng_init(&rng, run_i + p->opts->random_seed);
-    IGRAPH_FINALLY(igraph_rng_set_default, old_rng);
+    SE2_THREAD_CHECK_RETURN(
+      se2_rng_init(&rng, &old_rng, run_i + p->opts->random_seed), NULL);
     IGRAPH_FINALLY(igraph_rng_destroy, &rng);
+    IGRAPH_FINALLY(igraph_rng_set_default, &old_rng);
 
     SE2_THREAD_CHECK_RETURN(
       igraph_vector_int_init(&ic_store, p->n_nodes), NULL);
@@ -285,7 +286,7 @@ static void* se2_thread_bootstrap(void* parameters)
     SE2_THREAD_CHECK_RETURN(
       se2_core(p->graph, p->partition_store, partition_offset, p->opts), NULL);
 
-    igraph_rng_set_default(old_rng);
+    igraph_rng_set_default(&old_rng);
     igraph_rng_destroy(&rng);
     IGRAPH_FINALLY_CLEAN(2);
 
@@ -365,8 +366,10 @@ static igraph_error_t se2_bootstrap(se2_neighs const* graph,
 #ifdef SE2PAR
   pthread_t threads[opts->max_threads];
   pthread_mutex_t status_mutex[opts->max_threads];
-  struct se2_pthread_mutex_array status_mutex_holder = { .array = status_mutex,
-    .n = opts->max_threads };
+  struct se2_pthread_mutex_array status_mutex_holder = {
+    .array = status_mutex,
+    .n = opts->max_threads,
+  };
   for (igraph_integer_t i = 0; i < opts->max_threads; i++) {
     pthread_mutex_init(status_mutex + i, NULL);
   }
@@ -417,7 +420,7 @@ static igraph_error_t se2_bootstrap(se2_neighs const* graph,
       }
     }
 
-    if (igraph_allow_interruption(NULL) != IGRAPH_SUCCESS) {
+    if (igraph_allow_interruption()) {
       pthread_mutex_lock(&se2_error_mutex);
       se2_thread_errorcode = IGRAPH_INTERRUPTED;
       pthread_mutex_unlock(&se2_error_mutex);
@@ -431,7 +434,8 @@ static igraph_error_t se2_bootstrap(se2_neighs const* graph,
 #endif
 
   if (se2_thread_errorcode != IGRAPH_SUCCESS) {
-    IGRAPH_ERROR("", se2_thread_errorcode);
+    IGRAPH_FINALLY_FREE();
+    return se2_thread_errorcode;
   };
 
 #ifdef SE2PAR
@@ -555,9 +559,18 @@ static igraph_error_t se2_subgraph_from_community(se2_neighs const* origin,
     subgraph->weights = NULL;
   }
 
+  igraph_vector_int_t neighs;
+  if (!ISSPARSE(*origin)) {
+    IGRAPH_CHECK(igraph_vector_int_init(&neighs, se2_vcount(origin)));
+    IGRAPH_FINALLY(igraph_vector_int_destroy, &neighs);
+
+    for (igraph_integer_t i = 0; i < se2_vcount(origin); i++) {
+      VECTOR(neighs)[i] = i;
+    }
+  }
+
   for (igraph_integer_t i = 0; i < n_membs; i++) {
     igraph_integer_t node_id = VECTOR(*members)[i];
-    igraph_vector_int_t* neighs = &NEIGHBORS(*origin, node_id);
     igraph_vector_int_t* new_neighs = &NEIGHBORS(*subgraph, i);
     igraph_integer_t const n_neighs = N_NEIGHBORS(*origin, node_id);
     igraph_vector_t* w =
@@ -568,10 +581,14 @@ static igraph_error_t se2_subgraph_from_community(se2_neighs const* origin,
       IGRAPH_CHECK(igraph_vector_resize(w, n_neighs));
     }
 
+    if (ISSPARSE(*origin)) {
+      neighs = NEIGHBORS(*origin, node_id);
+    }
+
     igraph_integer_t count = 0;
     igraph_integer_t pos;
     for (igraph_integer_t j = 0; j < n_neighs; j++) {
-      if (igraph_vector_int_search(members, 0, VECTOR(*neighs)[j], &pos)) {
+      if (igraph_vector_int_search(members, 0, VECTOR(neighs)[j], &pos)) {
         VECTOR(*new_neighs)[count] = pos;
         if (HASWEIGHTS(*subgraph)) {
           VECTOR(*w)[count] = WEIGHT(*origin, node_id, j);
@@ -587,6 +604,11 @@ static igraph_error_t se2_subgraph_from_community(se2_neighs const* origin,
     }
   }
 
+  if (!ISSPARSE(*origin)) {
+    igraph_vector_int_destroy(&neighs);
+    IGRAPH_FINALLY_CLEAN(1);
+  }
+
   for (igraph_integer_t i = 0; i < n_membs; i++) {
     for (igraph_integer_t j = 0; j < N_NEIGHBORS(*subgraph, i); j++) {
       VECTOR(*subgraph->kin)
@@ -669,7 +691,7 @@ igraph_error_t speak_easy_2(
   }
 #endif
 
-  IGRAPH_CHECK(se2_reweigh(graph));
+  IGRAPH_CHECK(se2_reweigh(graph, opts->verbose));
 
   if (opts->verbose) {
     igraph_bool_t isweighted = false;
@@ -749,7 +771,7 @@ igraph_error_t speak_easy_2(
       IGRAPH_CHECK(se2_subgraph_from_community(graph, &subgraph, &member_ids));
       IGRAPH_FINALLY(se2_neighs_destroy, &subgraph);
 
-      IGRAPH_CHECK(se2_reweigh(&subgraph));
+      IGRAPH_CHECK(se2_reweigh(&subgraph, /* verbose */ false));
       IGRAPH_CHECK(se2_bootstrap(&subgraph, level, opts, &subgraph_memb));
 
       for (igraph_integer_t i = 0; i < igraph_vector_int_size(&subgraph_memb);
