@@ -72,31 +72,27 @@ static igraph_error_t se2_mean_link_weight(
   return IGRAPH_SUCCESS;
 }
 
-static igraph_error_t se2_weigh_diagonal(
-  se2_neighs* graph, igraph_bool_t is_skewed)
+static igraph_error_t se2_collect_sparse_diagonal(
+  se2_neighs* graph, igraph_vector_int_t* diag)
 {
   igraph_integer_t const n_nodes = se2_vcount(graph);
-  igraph_vector_int_t diagonal_edges;
-
-  IGRAPH_CHECK(igraph_vector_int_init(&diagonal_edges, n_nodes));
-  IGRAPH_FINALLY(igraph_vector_int_destroy, &diagonal_edges);
   for (igraph_integer_t i = 0; i < n_nodes; i++) {
     igraph_bool_t found_edge = false;
     for (igraph_integer_t j = 0; j < N_NEIGHBORS(*graph, i); j++) {
       if (NEIGHBOR(*graph, i, j) == i) {
         if (found_edge) { // Already found a diagonal.
           igraph_vector_int_remove(&NEIGHBORS(*graph, i), j);
-          N_NEIGHBORS(*graph, i)--;
+          VECTOR(*graph->sizes)[i]--;
           if (HASWEIGHTS(*graph)) {
             igraph_vector_remove(&WEIGHTS_IN(*graph, i), j);
           }
         } else {
           found_edge = true;
-          VECTOR(diagonal_edges)[i] = j;
+          VECTOR(*diag)[i] = j;
+          /* Importantly set to 0 so diagonal weights don't impact
+             calculation of mean link weight if skewed. Diagonal weights will
+             be written over anyway. */
           if (HASWEIGHTS(*graph)) {
-            /* Importantly set to 0 so diagonal weights don't impact
-               calculation of mean link weight if skewed. Diagonal weights will
-               be written over anyway. */
             igraph_vector_t* w = &WEIGHTS_IN(*graph, i);
             VECTOR(*w)[j] = 0;
           }
@@ -106,7 +102,7 @@ static igraph_error_t se2_weigh_diagonal(
 
     if (!found_edge) {
       IGRAPH_CHECK(igraph_vector_int_push_back(&NEIGHBORS(*graph, i), i));
-      VECTOR(diagonal_edges)[i] = N_NEIGHBORS(*graph, i)++;
+      VECTOR(*diag)[i] = VECTOR(*graph->sizes)[i]++;
       if (HASWEIGHTS(*graph)) {
         igraph_vector_t* w = &WEIGHTS_IN(*graph, i);
         IGRAPH_CHECK(igraph_vector_resize(w, N_NEIGHBORS(*graph, i)));
@@ -115,6 +111,26 @@ static igraph_error_t se2_weigh_diagonal(
     }
   }
 
+  return IGRAPH_SUCCESS;
+}
+
+static igraph_error_t se2_weigh_diagonal(
+  se2_neighs* graph, igraph_bool_t is_skewed, igraph_bool_t verbose)
+{
+  igraph_integer_t const n_nodes = se2_vcount(graph);
+  igraph_vector_int_t diagonal_edges;
+
+  IGRAPH_CHECK(igraph_vector_int_init(&diagonal_edges, n_nodes));
+  IGRAPH_FINALLY(igraph_vector_int_destroy, &diagonal_edges);
+
+  if (ISSPARSE(*graph)) {
+    IGRAPH_CHECK(se2_collect_sparse_diagonal(graph, &diagonal_edges));
+  } else {
+    for (igraph_integer_t i = 0; i < n_nodes; i++) {
+      VECTOR(diagonal_edges)[i] = i;
+    }
+  }
+
   if (!HASWEIGHTS(*graph)) {
     goto cleanup;
   }
@@ -124,8 +140,10 @@ static igraph_error_t se2_weigh_diagonal(
   IGRAPH_FINALLY(igraph_vector_destroy, &diagonal_weights);
 
   if (is_skewed) {
-    SE2_PUTS(
-      "High skew to edge weight distribution; reweighing main diagonal.");
+    if (verbose) {
+      SE2_PUTS(
+        "High skew to edge weight distribution; reweighing main diagonal.");
+    }
     IGRAPH_CHECK(se2_mean_link_weight(graph, &diagonal_weights));
   } else {
     igraph_vector_fill(&diagonal_weights, 1);
@@ -176,12 +194,14 @@ static void se2_reweigh_i(se2_neighs* graph)
   graph->total_weight /= max_magnitude_weight;
 }
 
-static igraph_error_t se2_add_offset(se2_neighs* graph)
+static igraph_error_t se2_add_offset(se2_neighs* graph, igraph_bool_t verbose)
 {
   igraph_integer_t const n_nodes = se2_vcount(graph);
   igraph_real_t offset = 0;
 
-  SE2_PUTS("adding very small offset to all edges");
+  if (verbose) {
+    SE2_PUTS("adding very small offset to all edges");
+  }
 
   for (igraph_integer_t i = 0; i < n_nodes; i++) {
     for (igraph_integer_t j = 0; j < N_NEIGHBORS(*graph, i); j++) {
@@ -238,15 +258,15 @@ void se2_recalc_degrees(se2_neighs* graph)
   }
 }
 
-igraph_error_t se2_reweigh(se2_neighs* graph)
+igraph_error_t se2_reweigh(se2_neighs* graph, igraph_bool_t verbose)
 {
   igraph_bool_t is_skewed = skewness(graph) >= 2;
 
   se2_reweigh_i(graph);
-  IGRAPH_CHECK(se2_weigh_diagonal(graph, is_skewed));
+  IGRAPH_CHECK(se2_weigh_diagonal(graph, is_skewed, verbose));
 
   if ((is_skewed) && (!se2_vector_list_has_negatives(graph))) {
-    se2_add_offset(graph);
+    se2_add_offset(graph, verbose);
   }
 
   se2_recalc_degrees(graph);
